@@ -18,26 +18,14 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-require 'logger'
+require 'console'
+require_relative 'task'
 
 module Async
-	# The Async Logger class.
-	class << self
-		# @attr logger [Logger] the global logger instance used by `Async`.
-		attr :logger
-	
-		# Set the default log level based on `$DEBUG` and `$VERBOSE`.
-		def default_log_level
-			if $DEBUG
-				Logger::DEBUG
-			elsif $VERBOSE
-				Logger::INFO
-			else
-				Logger::WARN
-			end
-		end
+	# @return the current logger, either the active tasks logger, or the global event console logger.
+	def self.logger
+		if task = Task.current?
+			task.logger
+		end || Console.logger
 	end
-
-	# Create the logger instance.
-	@logger = Logger.new($stderr).tap{|logger| logger.level = default_log_level}
 end
