@@ -27,8 +27,8 @@ require 'timers'
 require 'forwardable'
 
 module Async
-	# Raised if a timeout occurs on a specific Fiber. Handled gracefully by {Task}.
-	class TimeoutError < Exception
+	# Raised if a timeout occurs on a specific Fiber. Handled gracefully by `Task`.
+	class TimeoutError < StandardError
 	end
 	
 	# An asynchronous, cooperatively scheduled event reactor.
@@ -42,13 +42,13 @@ module Async
 		# - When invoked at the top level, will create and run a reactor, and invoke
 		# the block as an asynchronous task. Will block until the reactor finishes
 		# running.
-		def self.run(*args, &block)
+		def self.run(*args, **options, &block)
 			if current = Task.current?
 				reactor = current.reactor
 				
-				return reactor.async(*args, &block)
+				return reactor.async(*args, **options, &block)
 			else
-				reactor = self.new
+				reactor = self.new(**options)
 				
 				begin
 					return reactor.run(*args, &block)
@@ -58,11 +58,24 @@ module Async
 			end
 		end
 		
-		def initialize(parent = nil, selector: NIO::Selector.new)
+		def self.selector
+			if backend = ENV['ASYNC_BACKEND']&.to_sym
+				if NIO::Selector.backends.include?(backend)
+					return NIO::Selector.new(backend)
+				else
+					warn "Could not find ASYNC_BACKEND=#{backend}!"
+				end
+			end
+			
+			return NIO::Selector.new
+		end
+		
+		def initialize(parent = nil, selector: self.class.selector, logger: nil)
 			super(parent)
 			
 			@selector = selector
 			@timers = Timers::Group.new
+			@logger = logger
 			
 			@ready = []
 			@running = []
@@ -70,8 +83,12 @@ module Async
 			@stopped = true
 		end
 		
+		def logger
+			@logger ||= Console.logger
+		end
+		
 		def to_s
-			"<#{self.description} stopped=#{@stopped}>"
+			"\#<#{self.description} (#{@stopped ? 'stopped' : 'running'})>"
 		end
 		
 		# @attr stopped [Boolean]
@@ -81,6 +98,7 @@ module Async
 			@stopped
 		end
 		
+		# TODO Remove these in next major release. They are too confusing to use correctly.
 		def_delegators :@timers, :every, :after
 		
 		# Start an asynchronous task within the specified reactor. The task will be
@@ -89,21 +107,21 @@ module Async
 		#
 		# This is the main entry point for scheduling asynchronus tasks.
 		#
-		# @yield [Task] Executed within the asynchronous task.
-		# @return [Task] The task that was 
-		def async(*args, &block)
-			task = Task.new(self, &block)
+		# @yield [Task] Executed within the task.
+		# @return [Task] The task that was scheduled into the reactor.
+		def async(*args, **options, &block)
+			task = Task.new(self, **options, &block)
 			
 			# I want to take a moment to explain the logic of this.
 			# When calling an async block, we deterministically execute it until the
 			# first blocking operation. We don't *have* to do this - we could schedule
 			# it for later execution, but it's useful to:
-			# - Fail at the point of call where possible.
+			# - Fail at the point of the method call where possible.
 			# - Execute determinstically where possible.
-			# - Avoid overhead if no blocking operation is performed.
+			# - Avoid scheduler overhead if no blocking operation is performed.
 			task.run(*args)
 			
-			# Async.logger.debug "Initial execution of task #{fiber} complete (#{result} -> #{fiber.alive?})..."
+			# logger.debug "Initial execution of task #{fiber} complete (#{result} -> #{fiber.alive?})..."
 			return task
 		end
 		
@@ -113,7 +131,7 @@ module Async
 			
 			return monitor
 		end
-	
+		
 		# Stop the reactor at the earliest convenience. Can be called from a different thread safely.
 		# @return [void]
 		def stop
@@ -124,6 +142,7 @@ module Async
 		end
 		
 		# Schedule a fiber (or equivalent object) to be resumed on the next loop through the reactor.
+		# @param fiber [#resume] The object to be resumed on the next iteration of the run-loop.
 		def << fiber
 			@ready << fiber
 		end
@@ -136,6 +155,7 @@ module Async
 		end
 		
 		def finished?
+			# I'm not sure if checking `@running.empty?` is really required.
 			super && @ready.empty? && @running.empty?
 		end
 		
@@ -146,54 +166,54 @@ module Async
 			
 			@stopped = false
 			
-			# Allow the user to kick of the initial async tasks.
-			initial_task = async(*args, &block) if block_given?
+			initial_task = self.async(*args, &block) if block_given?
 			
-			@timers.wait do |interval|
-				# running used to correctly answer on `finished?`, and to reuse Array object.
-				@running, @ready = @ready, @running
-				if @running.any?
+			until @stopped
+				logger.debug(self) {"@ready = #{@ready} @running = #{@running}"}
+				
+				if @ready.any?
+					# running used to correctly answer on `finished?`, and to reuse Array object.
+					@running, @ready = @ready, @running
+					
 					@running.each do |fiber|
 						fiber.resume if fiber.alive?
 					end
+					
 					@running.clear
-
-					# if there are tasks ready to execute, don't sleep.
-					if @ready.any?
-						interval = 0
-					else
-						# The above tasks may schedule, cancel or affect timers in some way. We need to compute a new wait interval for the blocking selector call below:
-						interval = @timers.wait_interval
-					end
 				end
 				
-				# - nil: no timers
-				# - -ve: timers expired already
-				# -   0: timers ready to fire
-				# - +ve: timers waiting to fire
-				if interval && interval < 0
+				if @ready.empty?
+					interval = @timers.wait_interval
+				else
+					# if there are tasks ready to execute, don't sleep:
 					interval = 0
 				end
 				
-				# Async.logger.debug(self) {"Updating #{@children.count} children..."}
-				# As timeouts may have been updated, and caused fibers to complete, we should check this.
-				
-				# If there is nothing to do, then finish:
-				if !interval && self.finished?
-					return initial_task
+				# If there is no interval to wait (thus no timers), and no tasks, we could be done:
+				if interval.nil?
+					if self.finished?
+						# If there is nothing to do, then finish:
+						return initial_task
+					end
+				elsif interval < 0
+					# We have timers ready to fire, don't sleep in the selctor:
+					interval = 0
 				end
 				
-				# Async.logger.debug(self) {"Selecting with #{@children.count} fibers interval = #{interval.inspect}..."}
+				logger.debug(self) {"Selecting with #{@children&.size} children with interval = #{interval ? interval.round(2) : 'infinite'}..."}
 				if monitors = @selector.select(interval)
 					monitors.each do |monitor|
 						monitor.value.resume
 					end
 				end
-			end until @stopped
+				
+				@timers.fire
+			end
 			
 			return initial_task
 		ensure
-			Async.logger.debug(self) {"Exiting run-loop because #{$! ? $!.inspect : 'finished'}."}
+			logger.debug(self) {"Exiting run-loop because #{$! ? $! : 'finished'}."}
+			
 			@stopped = true
 		end
 	
@@ -201,10 +221,9 @@ module Async
 		# 
 		# @return [void]
 		def close
-			@children.each(&:stop)
+			@children&.each(&:stop)
 			
 			# TODO Should we also clear all timers?
-			
 			@selector.close
 			@selector = nil
 		end
@@ -231,25 +250,29 @@ module Async
 			timer.cancel if timer
 		end
 		
-		# Invoke the block, but after the timeout, raise {TimeoutError} in any
-		# currenly blocking operation.
-		# @param duration [Integer] The time in seconds, in which the task should 
+		# Invoke the block, but after the specified timeout, raise {TimeoutError} in any currenly blocking operation. If the block runs to completion before the timeout occurs or there are no non-blocking operations after the timeout expires, the code will complete without any exception.
+		# @param duration [Numeric] The time in seconds, in which the task should 
 		#   complete.
-		def timeout(duration)
-			backtrace = caller
+		def with_timeout(timeout, exception = TimeoutError)
 			fiber = Fiber.current
 			
-			timer = self.after(duration) do
+			timer = self.after(timeout) do
 				if fiber.alive?
-					error = TimeoutError.new("execution expired")
-					error.set_backtrace backtrace
+					error = exception.new("execution expired")
 					fiber.resume error
 				end
 			end
 			
-			yield
+			yield timer
 		ensure
 			timer.cancel if timer
 		end
+		
+		# TODO remove
+		def timeout(*args, &block)
+			warn "#{self.class}\#timeout(...) is deprecated, use #{self.class}\#with_timeout(...) instead."
+			
+			with_timeout(*args, &block)
+		end
 	end
 end
