@@ -18,6 +18,7 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+require 'async/rspec'
 require 'async/condition'
 
 require_relative 'condition_examples'
@@ -28,7 +29,6 @@ RSpec.describe Async::Condition do
 	it 'should continue after condition is signalled' do
 		task = reactor.async do
 			subject.wait
-			#puts "Got #{value}"
 		end
 		
 		expect(task.status).to be :running
@@ -41,5 +41,30 @@ RSpec.describe Async::Condition do
 		task.stop
 	end
 	
+	it 'can stop nested task' do
+		producer = nil
+		
+		consumer = reactor.async do |task|
+			condition = Async::Condition.new
+			
+			producer = task.async do |subtask|
+				subtask.yield
+				condition.signal
+				subtask.sleep(10)
+			end
+			
+			condition.wait
+			expect do
+				producer.stop
+			end.to_not raise_error
+		end
+		
+		consumer.wait
+		producer.wait
+		
+		expect(producer.status).to be :stopped
+		expect(consumer.status).to be :complete
+	end
+	
 	it_behaves_like Async::Condition
 end
