@@ -1,4 +1,4 @@
-# Copyright, 2018, by Samuel G. D. Williams. <http://www.codeotaku.com>
+# Copyright, 2017, by Samuel G. D. Williams. <http://www.codeotaku.com>
 # 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -18,32 +18,46 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
-RSpec.describe Async.logger do
-	describe '::default_log_level' do
-		let!(:debug) {$DEBUG}
-		after {$DEBUG = debug}
+require 'async'
+require 'async/logger'
+require 'console/capture'
+
+RSpec.describe 'Async.logger' do
+	let(:name) {"nested"}
+	let(:message) {"Talk is cheap. Show me the code."}
+	
+	let(:capture) {Console::Capture.new}
+	let(:logger) {Console::Logger.new(capture, name: name)}
+	
+	it "can use nested logger" do
+		Async(logger: logger) do |task|
+			expect(task.logger).to be == logger
+			logger.warn message
+		end.wait
 		
-		let!(:verbose) {$VERBOSE}
-		after {$VERBOSE = verbose}
-		
-		it 'should set default log level' do
-			$DEBUG = false
-			$VERBOSE = false
+		expect(capture.events.last).to include({
+			severity: :warn,
+			name: name,
+			subject: message,
+		})
+	end
+	
+	it "can change nested logger" do
+		Async do |parent|
+			parent.async(logger: logger) do |task|
+				expect(task.logger).to be == logger
+				expect(Async.logger).to be == logger
+			end.wait
+		end.wait
+	end
+	
+	it "can use parent logger" do
+		Async(logger: logger) do |parent|
+			child = parent.async{|task| task.yield}
 			
-			expect(Async.default_log_level).to be == Logger::WARN
-		end
-		
-		it 'should set default log level based on $DEBUG' do
-			$DEBUG = true
-			
-			expect(Async.default_log_level).to be == Logger::DEBUG
-		end
-		
-		it 'should set default log level based on $VERBOSE' do
-			$DEBUG = false
-			$VERBOSE = true
-			
-			expect(Async.default_log_level).to be == Logger::INFO
-		end
+			expect(parent.logger).to be == logger
+			expect(child.logger).to be == logger
+			expect(Async.logger).to be == logger
+		end.wait
 	end
 end
