@@ -18,13 +18,13 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+require 'async'
 require 'async/queue'
+require 'async/rspec'
 
 require_relative 'condition_examples'
 
-RSpec.describe Async::Queue do
-	include_context Async::RSpec::Reactor
-	
+RSpec.shared_context Async::Queue do
 	it 'should process items in order' do
 		reactor.async do |task|
 			10.times do |i|
@@ -37,4 +37,54 @@ RSpec.describe Async::Queue do
 			expect(subject.dequeue).to be == j
 		end
 	end
+	
+	it 'can dequeue items asynchronously' do
+		reactor.async do |task|
+			subject << 1
+			subject << nil
+		end
+		
+		subject.async do |task, item|
+			expect(item).to be 1
+		end
+	end
+end
+
+RSpec.describe Async::Queue do
+	include_context Async::RSpec::Reactor
+	
+	it_behaves_like Async::Queue
+end
+
+RSpec.describe Async::LimitedQueue do
+	include_context Async::RSpec::Reactor
+	
+	it_behaves_like Async::Queue
+	
+	it 'should become limited' do
+		expect(subject).to_not be_limited
+		subject.enqueue(10)
+		expect(subject).to be_limited
+	end
+
+	it 'should resume waiting tasks in order' do
+		total_resumed = 0
+		total_dequeued = 0
+		Async do |producer|
+			10.times do
+				producer.async do
+					subject.enqueue('foo')
+					total_resumed += 1
+				end
+			end
+		end
+		Async do |consumer|
+			10.times do
+				subject.dequeue
+				total_dequeued += 1
+
+				expect(total_resumed).to be == total_dequeued
+			end
+		end
+	end
 end
