@@ -18,6 +18,9 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+require 'async'
+require 'benchmark/ips'
+
 RSpec.describe Async::Reactor do
 	describe '#run' do
 		it "can run tasks on different fibers" do
@@ -34,6 +37,14 @@ RSpec.describe Async::Reactor do
 		end
 	end
 	
+	describe '#close' do
+		it "can close empty reactor" do
+			subject.close
+			
+			expect(subject).to be_closed
+		end
+	end
+	
 	describe '#stop' do
 		it "can be stop reactor" do
 			state = nil
@@ -52,6 +63,8 @@ RSpec.describe Async::Reactor do
 			subject.run
 			
 			expect(state).to be == :started
+			
+			subject.close
 		end
 		
 		it "can stop reactor from different thread" do
@@ -70,31 +83,33 @@ RSpec.describe Async::Reactor do
 			subject.run
 			
 			thread.join
+			
 			expect(subject).to be_stopped
 		end
 	end
+	
 	it "can't return" do
 		expect do
-			Async::Reactor.run do |task|
+			Async do |task|
 				return
-			end
-		end.to raise_error(LocalJumpError)
+			end.wait
+		end.to raise_exception(LocalJumpError)
 	end
 	
 	it "is closed after running" do
 		reactor = nil
 		
-		Async::Reactor.run do |task|
+		Async do |task|
 			reactor = task.reactor
 		end
 		
 		expect(reactor).to be_closed
 		
-		expect{reactor.run}.to raise_error(RuntimeError, /closed/)
+		expect{reactor.run}.to raise_exception(RuntimeError, /closed/)
 	end
 	
 	it "should return a task" do
-		result = Async::Reactor.run do |task|
+		result = Async do |task|
 		end
 		
 		expect(result).to be_kind_of(Async::Task)
@@ -118,7 +133,7 @@ RSpec.describe Async::Reactor do
 		end
 	end
 	
-	describe '#timeout' do
+	describe '#with_timeout' do
 		let(:duration) {1}
 		
 		it "stops immediately" do
@@ -127,7 +142,7 @@ RSpec.describe Async::Reactor do
 			described_class.run do |task|
 				condition = Async::Condition.new
 				
-				task.timeout(duration) do
+				task.with_timeout(duration) do
 					task.async do
 						condition.wait
 					end
@@ -144,6 +159,35 @@ RSpec.describe Async::Reactor do
 			
 			expect(duration).to be < 0.1
 		end
+		
+		let(:timeout_class) {Class.new(RuntimeError)}
+		
+		it "raises specified exception" do
+			expect do
+				described_class.run do |task|
+					task.with_timeout(0.0, timeout_class) do
+						task.sleep(1.0)
+					end
+				end.wait
+			end.to raise_exception(timeout_class)
+		end
+		
+		it "should be fast to use timeouts" do
+			Benchmark.ips do |x|
+				x.report('Reactor#with_timeout') do |repeats|
+					Async do |task|
+						reactor = task.reactor
+						
+						repeats.times do
+							reactor.with_timeout(1) do
+							end
+						end
+					end
+				end
+				
+				x.compare!
+			end
+		end
 	end
 	
 	describe '#to_s' do
